#include "serial.h"

#ifdef _UART0_ENABLE
//-----------------------------------------------------------------------------
// Serial Prot0 Utility Fuction Routine
void U0_Init(unsigned char baud)
{
	U0R_CSRA |= UART_DB_BAUDRATE;

	U0_SetBaud(baud);
	
	// enable serial uart & receiver
	u0_EnableTx();
	u0_EnableRx(); 
	
  	fdevopen(U0_PutByte,  U0_GetByte);
}

void U0_SetBaud(unsigned char baud)
{
	unsigned char baud_rate = 0;
	
	switch (baud)
	{
		case BAUD_9600:
			baud_rate = BAUDRATE_VAL_9600;
			break;
		case BAUD_14400:
			baud_rate = BAUDRATE_VAL_14400;
			break;		
		case BAUD_19200:
			baud_rate = BAUDRATE_VAL_19200;
			break;		
		case BAUD_28800:
			baud_rate = BAUDRATE_VAL_28800;
			break;		
		case BAUD_38400:
			baud_rate = BAUDRATE_VAL_38400;
			break;		
		case BAUD_57600:
			baud_rate = BAUDRATE_VAL_57600;
			break;		
		case BAUD_76800:
			baud_rate = BAUDRATE_VAL_76800;
			break;		
		case BAUD_115200:
			baud_rate = BAUDRATE_VAL_115200;
			break;		
		default :
			baud_rate = BAUDRATE_VAL_DEFAULT;			
			break;
	}	
	
	U0R_UBRRL = baud_rate;
}

unsigned char U0_GetByte(void)
{
	u0_WaitForReceive();
	
	return u0_RxData();
}

void U0_PutByte(unsigned char Data)
{
	u0_WaitForSend();
	
	u0_TxData(Data);
}

void U0_PutBuf(unsigned char *str, unsigned char len)
{
	unsigned char i =0;
	
    for(i=0;i<len;i++)
    {
        U0_PutByte(*(str+i));
    }
}

void U0_PutStr(unsigned char *str)
{
    while(*str != 0)
    {
        U0_PutByte(*str);
        str++;
    }
}


unsigned char U0_IsGetByte(void)
{
	return u0_IsReceiveData();
}
//------------------------------------------------------------------------------
#endif	//end UART0 ENABLE


#ifdef _UART1_ENABLE
//-----------------------------------------------------------------------------
// USART Fuction Declaration
// Serial Prot1 Utility Fuction Routine
void U1_Init(unsigned char baud)
{
	U1_SetBaud(baud);
	
	// enable serial uart & receiver
	u1_EnableTx();
	u1_EnableRx(); 
	
	//at 16Mhz
	UCSR1A |= UART_DB_BAUDRATE;

  	fdevopen(U1_PutByte,  U1_GetByte);
}

void U1_SetBaud(unsigned char baud)
{
	switch (baud)
	{
		case BAUD_9600:
			UBRR1L = BAUDRATE_VAL_9600;
			break;
		case BAUD_14400:
			UBRR1L = BAUDRATE_VAL_14400;
			break;		
		case BAUD_19200:
			UBRR1L = BAUDRATE_VAL_19200;
			break;		
		case BAUD_28800:
			UBRR1L = BAUDRATE_VAL_28800;
			break;		
		case BAUD_38400:
			UBRR1L = BAUDRATE_VAL_38400;
			break;		
		case BAUD_57600:
			UBRR1L = BAUDRATE_VAL_57600;
			break;		
		case BAUD_76800:
			UBRR1L = BAUDRATE_VAL_76800;
			break;		
		case BAUD_115200:
			UBRR1L = BAUDRATE_VAL_115200;
			break;		
		default :
			UBRR1L = BAUDRATE_VAL_DEFAULT;			
			break;
	}	
}

unsigned char U1_GetByte(void)
{
	u1_WaitForReceive();
	
	return u1_RxData();
}

void U1_PutByte(unsigned char Data)
{
	u1_WaitForSend();
	
	u1_TxData(Data);
}

void U1_PutBuf(unsigned char *str, unsigned char len)
{
	unsigned char i =0;
	
    for(i=0;i<len;i++)
    {
        U1_PutByte(*(str+i));
    }
}

void U1_PutStr(unsigned char *str)
{
    while(*str != 0)
    {
        U1_PutByte(*str);
        str++;
    }
}

unsigned char U1_IsGetByte(void)
{
	return u1_IsReceiveData();
}
//------------------------------------------------------------------------------
#endif	//end _UART1_ENABLE
