#include "system.h"
//------------------------------------------------------------------------------
// UART Defination
//------------------------------------------------------------------------------
#define BAUD_9600							0
#define BAUD_14400							1
#define BAUD_19200							2
#define BAUD_28800							3
#define BAUD_38400							4		
#define BAUD_57600							5		
#define BAUD_76800							6		
#define BAUD_115200							7	


#define u0_setbaud(bdr)  	( U0R_UBRRH = (unsigned char)((((unsigned long)__SYS_CLK__*1000L)/((unsigned long)bdr*16)-1)>>8),\
                             U0R_UBRRL = (unsigned char)(((unsigned long)__SYS_CLK__*1000 )/((unsigned long)bdr*16)-1)    )
  
#if (__SYS_CLK__ == AVR_CLK_16MHZ)	
#define BAUDRATE_VAL_9600					207
#define BAUDRATE_VAL_19200					103
#define BAUDRATE_VAL_14400					138
#define BAUDRATE_VAL_28800					68
#define BAUDRATE_VAL_38400					51
#define BAUDRATE_VAL_57600					34
#define BAUDRATE_VAL_76800					25
#define BAUDRATE_VAL_115200					16
#elif (__SYS_CLK__ == AVR_CLK_8MHZ)	
#define BAUDRATE_VAL_9600					103
#define BAUDRATE_VAL_14400					68
#define BAUDRATE_VAL_19200					51
#define BAUDRATE_VAL_28800					34
#define BAUDRATE_VAL_38400					25
#define BAUDRATE_VAL_57600					16
#define BAUDRATE_VAL_76800					12
#define BAUDRATE_VAL_115200					8
#elif (__SYS_CLK__ == AVR_CLK_73728KHZ)
#define BAUDRATE_VAL_9600					95
#define BAUDRATE_VAL_14400					63
#define BAUDRATE_VAL_19200					47
#define BAUDRATE_VAL_28800					31
#define BAUDRATE_VAL_38400					23
#define BAUDRATE_VAL_57600					15
#define BAUDRATE_VAL_76800					11
#define BAUDRATE_VAL_115200					7
#endif

#define UART_DB_BAUDRATE					0x02
#define BAUDRATE_VAL_DEFAULT				BAUDRATE_VAL_57600

#ifdef _UART0_ENABLE
//------------------------------------------------------
//Atmega128 Serial Register
#if (__MCU_TYPE__ == AVR_TYPE_MEGA128)	
#define U0R_DATA							UDR0
#define U0R_CSRA							UCSR0A
#define U0R_CSRB							UCSR0B
#define U0R_UBRR							UBRR0L
#define U0R_UBRRH							UBRR0H
#define U0R_UBRRL							UBRR0L
#elif(__MCU_TYPE__ == AVR_TYPE_MEGA8)	
//------------------------------------------------------
//Atmega8 Serial Register
#define U0R_DATA							UDR
#define U0R_CSRA							UCSRA
#define U0R_CSRB							UCSRB
#define U0R_UBRRH							UBRRH
#define U0R_UBRRL							UBRRL
#define U0R_UBRR							UBRRL
#elif (__MCU_TYPE__ == AVR_TYPE_MEGA163)	
//------------------------------------------------------
//Atmega163 Serial Register
#define U0R_DATA							UDR
#define U0R_CSRA							UCSRA
#define U0R_CSRB							UCSRB
#define U0R_UBRRH							UBRRH
#define U0R_UBRRL							UBRR
#define U0R_UBRR							UBRR
#endif
//------------------------------------------------------

#define u0_WaitForReceive()					while(!(U0R_CSRA & 0x80))
#define u0_RxData()		 					(U0R_DATA)

#define u0_TxData(Data)						(U0R_DATA = Data)
#define u0_WaitForSend()					while(!(U0R_CSRA & 0x20))

#define u0_IsReceiveData()					(U0R_CSRA & 0x80)

#define u0_EnableRxIsr()					(U0R_CSRB |= 0x80) 
#define u0_EnableTxIsr()					(U0R_CSRB |= 0x40) 
#define u0_EnableRx()						(U0R_CSRB |= 0x10)
#define u0_EnableTx()						(U0R_CSRB |= 0x08)

//-----------------------------------------------------------------------------
// USART Fuction Declaration
unsigned char U0_IsGetByte(void);
void U0_Init(unsigned char baud);
void U0_SetBaud(unsigned char baud);
unsigned char U0_GetByte(void);
void U0_PutByte(unsigned char Data);
void U0_PutBuf(unsigned char *str, unsigned char len);
//------------------------------------------------------------------------------
#endif	//end UART0 ENABLE


#ifdef _UART1_ENABLE

#define U1_printf							printf

#define u1_WaitForReceive()					while(!(UCSR1A & 0x80))
#define u1_RxData()		 					(UDR1)

#define u1_TxData(Data)						(UDR1 = Data)
#define u1_WaitForSend()					while(!(UCSR1A & 0x20))

#define u1_IsReceiveData()					((UCSR1A & 0x80))

#define u1_EnableRxIsr()					(UCSR1B |= 0x80) 
#define u1_EnableTxIsr()					(UCSR1B |= 0x40) 
#define u1_EnableRx()						(UCSR1B |= 0x10)
#define u1_EnableTx()						(UCSR1B |= 0x08)

//-----------------------------------------------------------------------------
// USART Fuction Declaration
unsigned char U1_IsGetByte(void);
void U1_Init(unsigned char baud);
void U1_SetBaud(unsigned char baud);
unsigned char U1_GetByte(void);
void U1_PutByte(unsigned char Data);
void U1_PutStr(unsigned char *str);
void U1_PutBuf(unsigned char *str, unsigned char len);
//------------------------------------------------------------------------------
#endif	//end _UART1_ENABLE
